/**
 * @author zhixin wen <wenzhixin2010@gmail.com>
 * extensions: https://github.com/vitalets/x-editable
 */
 import $P from '../../../../../src/utils/common.js'

(function($) {

    'use strict';

    $.extend($.fn.bootstrapTable.defaults, {
        editable: true,
        onEditableInit: function() {
            return false;
        },
        onEditableSave: function(field, row, oldValue, $el,column,options,index) {
            if(row[field] != oldValue){
                row["dirty"] = true;
                row[field+"_dirty"] = true;
                if(row["orikey_"+field] == undefined){
                    row["orikey_"+field]=$P.gv(oldValue,"");
                }
                if((typeof options.columnChange) == 'function'){
                    options.columnChange.call(this,field, row, oldValue, $el,column,options,index);
                }
            }
            return false;
        },
        onEditableShown: function(field, row, $el, editable) {
            return false;
        },
        onEditableHidden: function(field, row, $el, reason) {
            return false;
        },
        onEdititleSave:function (field, value,params, $el, column,options) {
            return false;
        }
    });

    $.extend($.fn.bootstrapTable.Constructor.EVENTS, {
        'editable-init.bs.table': 'onEditableInit',
        'editable-save.bs.table': 'onEditableSave',
        'editable-shown.bs.table': 'onEditableShown',
        'editable-hidden.bs.table': 'onEditableHidden',
        'edititle-save.bs.table': 'onEdititleSave',

    });

    var BootstrapTable = $.fn.bootstrapTable.Constructor,
        _initTable = BootstrapTable.prototype.initTable,
        _resetView = BootstrapTable.prototype.resetView,
        _initBody = BootstrapTable.prototype.initBody;

    BootstrapTable.prototype.resetView = function() {
        var that = this;
        var gridId = that.$el.attr("id");
        _resetView.apply(this, Array.prototype.slice.apply(arguments));

        if (!this.options.editable) {
            return;
        }

        $.each(that.columns, function(i, column) {
            if (!(column.editable || column.edititle || column.titleFilter)) {
                return;
            }

            if (column.edititle || column.titleFilter) {
                var titleId = gridId+"_"+column.field;
                that.$header.find("th[data-field="+column.field+"]").find("div").eq(0).attr("id",titleId).css("cursor","pointer");
                column["titleId"]=titleId;
            }
        });

        setTimeout(function(){
            $.each(that.columns, function(i, column) {
                if (column.titleFilter) {
                    $("#"+column["titleId"]).find("#titleFilter").off('click').on('click', function(e, params) {
                        $P.showGridTitleFilter(that,column,this);
                    });
                }

                // if (!column.editable) {
                //     return;
                // }



                if (column.edititle) {
                    var el = $("#"+column["titleId"]).find("#editspan");
                    el.editable("destroy");
                    setTimeout(function () {
                        el.editable(column.edititle).off('save').on('save', function(e, params) {
                            that.trigger('edititle-save', column.field,params.submitValue,params, $(this),column,that.options);
                            $P.recordLog({
                                BILL_NO:$P.getPageId(),
                                OP_TYPE:$P.gv(document.title),
                                CONTENT:"修改"+$P.gv(column.title,column.field),
                                MODULE:"操作日志"
                            });
                        });
                    },200)

                }
            });
        },500);

    }


    BootstrapTable.prototype.initTable = function() {
        var that = this;
        _initTable.apply(this, Array.prototype.slice.apply(arguments));


        if (!this.options.editable) {
            return;
        }

        $.each(this.columns, function(i, column) {
            if (!column.editable) {
                return;
            }

            var editableOptions = {},
                editableDataMarkup = [],
                editableDataPrefix = 'editable-';

            var processDataOptions = function(key, value) {
                // Replace camel case with dashes.
                var dashKey = key.replace(/([A-Z])/g, function($1) {
                    return "-" + $1.toLowerCase();
                });
                if (dashKey.slice(0, editableDataPrefix.length) == editableDataPrefix) {
                    var dataKey = dashKey.replace(editableDataPrefix, 'data-');
                    editableOptions[dataKey] = value;
                }
            };

            $.each(that.options, processDataOptions);

            column.formatter = column.formatter || function(value, row, index) {
                return value;
            };
            column._formatter = column._formatter ? column._formatter : column.formatter;
            column.formatter = function(value, row, index) {
                var result = column._formatter ? column._formatter(value, row, index) : value;
                if(result == undefined){result = ""};
                $.each(column, processDataOptions);

                $.each(editableOptions, function(key, value) {
                    editableDataMarkup.push(' ' + key + '="' + value + '"');
                });

                var _dont_edit_formatter = false;
                if (column.editable.hasOwnProperty('noeditFormatter')) {
                    _dont_edit_formatter = column.editable.noeditFormatter(value, row, index);
                }

                if (_dont_edit_formatter === false) {
                    if(column.editable.is_rule == 1){
                        return ['<a href="javascript:void(0)"',
                            ' data-name="' + column.field + '"',
                            ' id="' + column.field + '"',
                            ' data-pk="' + row[that.options.idField] + '"',
                            ' data-value="' + result + '"',
                            ' data-text="' + $P.gv(row[that.field+"_TEXT"],result) + '"',
                            editableDataMarkup.join(''),
                            '>'+ result+'</a>'
                        ].join('');
                    }else{
                        var title = result;
                        var _color = "";
                        var _bgColor = "";

                        if($P.getObj("COL_COLOR_CONFIG_"+column.field) != undefined){
                            var rs = $P.getGridColColorResult(value, row, index,column.field,$P.getObj("COL_COLOR_CONFIG_"+column.field) );
                            if(rs.bFitCon == true){
                                _color = $P.gv(rs.color);
                                _bgColor = $P.gv(rs.bgcolor);
                                title = $P.gv(rs.title);
                            }
                        };
                        if(column.editable.type == "date"){
                            return  ['<a href="javascript:void(0)"',
                                ' data-name="' + column.field + '"',
                                ' id="' + column.field + '"',
                                ' data-pk="' + row[that.options.idField] + '"',
                                ' data-value="' + result + '"',
                                ' data-text="' + $P.gv(row[that.field+"_TEXT"],result) + '"',
                                ' title="' + title + '"',
                                column.editable.fixedWidth >0 ?' style="'+_color+_bgColor+'width:'+column.editable.fixedWidth+'px" class="single_slh_a" ':' style="'+_color+_bgColor+'"',
                                editableDataMarkup.join(''),
                                '>'+ result+ '</a>'
                            ].join('');
                        }else{
                            return  ['<a href="javascript:void(0)"',
                                ' data-name="' + column.field + '"',
                                ' id="' + column.field + '"',
                                ' data-pk="' + row[that.options.idField] + '"',
                                ' data-value="' + result + '"',
                                ' data-text="' + $P.gv(row[that.field+"_TEXT"],result) + '"',
                                ' title="' + title + '"',
                                column.editable.fixedWidth >0 ?' style="'+_color+_bgColor+'width:'+column.editable.fixedWidth+'px" class="single_slh_a" ':' style="'+_color+_bgColor+'"',
                                editableDataMarkup.join(''),
                                '>'+ '</a>'
                            ].join('');
                        }
                    }
                } else {
                    return _dont_edit_formatter;
                }

            };
        });
    };

    BootstrapTable.prototype.initBody = function() {
        var that = this;
        _initBody.apply(this, Array.prototype.slice.apply(arguments));

        if (!this.options.editable) {
            return;
        }

        $.each(this.columns, function(i, column) {
            if (!column.editable) {
                return;
            }

            if(column.editable["escape"]){
                column["escape"] = true;
            }

            that.$body.find('a[data-name="' + column.field + '"]').each(function (index,el) {
                $(el).parent().click(function () {
                    debugger
                    setTimeout(function(){
                        $(el).editable('show');
                    },50);
                })

                var data = that.getData(),
                    index = $(this).parents('tr[data-index]').data('index'),
                    row = data[index];

                if(column.canEditFun != undefined && (typeof column.canEditFun) == 'function'){
                    if(!column.canEditFun.call(this,index,row,data,column)){
                        column.editable.disabled = true;
                    }else{
                        column.editable.disabled = false;
                    }
                }

                if(column.paramConfig["delay"] == true && (typeof  that.options["postParamFun"]) == 'function'){
                     var bTempKey = (typeof  that.options["tempKeyFun"]) == 'function';
                     column.editable["source"] = $P.getUrlData(column.paramConfig["url"],
                         that.options["postParamFun"].call(that,column.field,index,row,data),
                         column.paramConfig["config"],
                         bTempKey ? that.options["tempKeyFun"].call(that,column.field,index,row,data) : column.field);
                }

                column.editable["otherConfig"]={
                    row:row,
                    index:index,
                    data:data,
                    field:column.field,
                    column:column
                }

                $(this).editable(column.editable).off('save').on('save', function(e, params) {
                    var oldValue = row[column.field];
                    $(this).data('value', params.submitValue);
                    //console.log(column.field,params);
                    row[column.field] = params.submitValue;
                    that.trigger('editable-save', column.field, row, oldValue, $(this),column,that.options,index);
                    that.resetFooter();
                }).off('shown').on('shown', function(e, editable) {
                    that.trigger('editable-shown', column.field, row, $(this), editable);
                }).off('hidden').on('hidden', function(e, reason) {
                    that.trigger('editable-hidden', column.field, row, $(this), reason);
                });
            });

           /*that.$body.find('a[data-name="' + column.field + '"]').editable(column.editable).off('save').on('save', function(e, params) {
                    var data = that.getData(),
                        index = $(this).parents('tr[data-index]').data('index'),
                        row = data[index],
                        oldValue = row[column.field];

                    $(this).data('value', params.submitValue);
                    row[column.field] = params.submitValue;
                    that.trigger('editable-save', column.field, row, oldValue, $(this),column,that.options,index);
                    that.resetFooter();
                }).off('shown').on('shown', function(e, editable) {
                    var data = that.getData(),
                        index = $(this).parents('tr[data-index]').data('index'),
                        row = data[index];

                    that.trigger('editable-shown', column.field, row, $(this), editable);
                }).off('hidden').on('hidden', function(e, reason) {
                    var data = that.getData(),
                        index = $(this).parents('tr[data-index]').data('index'),
                        row = data[index];

                    that.trigger('editable-hidden', column.field, row, $(this), reason);
                });*/
        });
        this.trigger('editable-init');
    };

})(jQuery);


